from datetime import datetime, timedelta
from random import randint
from typing import TYPE_CHECKING, List, Optional

from awesome_git_mosaic.charmaps.basic.basic_charmap import BasicCharmap

if TYPE_CHECKING:
    from awesome_git_mosaic.charmaps.charmap import Charmap


class GitMosaicAdapter:
    def __init__(self, charmap: Optional["Charmap"] = None):
        self.charmap = charmap or BasicCharmap()

    def output(
        self,
        message: str,
        reference_day: Optional[datetime] = None,
        with_spaces: bool = True,
        inverted: bool = False,
    ) -> List[datetime]:
        reference_day = reference_day or datetime.today()
        lines = self.charmap.translate(message, with_spaces, inverted)

        output = []
        y_offset = 0
        for line in lines:
            x_offset = 0
            for char in line:
                if self.charmap.is_pixel(char):
                    output.append(
                        self._timestamp_for_pixel(x_offset, y_offset, reference_day)
                    )
                x_offset += 1
            y_offset += 1

        return output

    def _timestamp_for_pixel(
        self, x_offset: int, y_offset: int, reference_day: datetime
    ) -> datetime:
        last_sunday = reference_day - timedelta(days=reference_day.weekday() + 1)
        top_left_square = last_sunday - timedelta(weeks=51)

        target_date = (
            top_left_square
            + timedelta(weeks=x_offset)
            + timedelta(days=y_offset)
            + timedelta(microseconds=randint(0, 300))
        )
        return target_date



from datetime import datetime, timedelta

from awesome_git_mosaic.gateways.git.git_gateway import GitGateway


class MakeCommit:
    def __init__(self, git_gateway: GitGateway = None):
        self.git_gateway = git_gateway or GitGateway()

    def make_commit(self):
        return NotImplemented



from os import path
from typing import Optional

from unidecode import unidecode

from awesome_git_mosaic.charmaps.charmap import Charmap

CHARMAP_PIXEL = "o"


class BasicCharmap(Charmap):
    def __init__(
        self,
        model_file: Optional[str] = None,
        char_width: int = 5,
        char_height: int = 7,
        char_list: str = " abcdefghijklmnopqrstuvwxyz0123456789#",
    ) -> None:
        self.char_height = char_height
        if not model_file:
            current_dir = path.dirname(path.abspath(__file__))
            model_file = path.join(current_dir, "char_model.txt")
        self.chars = self._load_char_model(
            model_file, char_width, char_height, char_list
        )

    def translate(
        self, string: str, with_spaces: bool = True, inverted: bool = False
    ) -> list:
        string = unidecode(string).lower()
        mapped_chars = [self.chars[c] for c in string]

        space = " " if with_spaces else ""

        output = []
        for line in range(self.char_height):
            line_str = space.join(["".join(char[line]) for char in mapped_chars])
            if inverted:
                line_str = line_str.translate(
                    str.maketrans(f"{CHARMAP_PIXEL} ", f" {CHARMAP_PIXEL}")
                )
            output.append(line_str)

        return output

    def is_pixel(self, char: str) -> bool:
        return char == CHARMAP_PIXEL

    def _load_char_model(
        self, model_file: str, char_width: int, char_height: int, char_list: str
    ) -> dict[str, list[list]]:
        f = open(model_file, "r")
        lines = f.read().splitlines(keepends=False)
        f.close()

        char_map: dict[str, list[list]] = {}

        for char_index, char in enumerate(char_list):
            char_map[char] = []

            for i in range(char_height):
                char_line = i + char_index + (char_height * char_index)

                char_map[char].append(list(lines[char_line][:char_width]))

        return char_map



import sys

from . import write


def main():
    if len(sys.argv) > 1:
        write(sys.argv[1], 45, 1, True)
    else:
        print("You need to give something to write")


if __name__ == "__main__":
    main()



import subprocess
from datetime import datetime
from typing import Optional


class GitGateway:
    def add(self, path: str = "."):
        return self._git("add", path)

    def commit(self, message: str, date: Optional[datetime] = None):
        commit_args = [f'-m"{message}"']
        if date:
            commit_args += ["--date", f'"{date.ctime()}"']
        return self._git("commit", *commit_args)

    def push(self, branch: Optional[str] = None):
        if not branch:
            branch = self._current_branch()

        return self._git("push", "origin", branch)

    def disable_garbage_collector(self):
        return self._git("config", "--local", "gc.auto", "0")

    def enable_garbage_collector(self):
        return self._git("config", "--local", "gc.auto", "1")

    def _current_branch(self):
        return self._git("rev-parse", "--abbrev-ref", "HEAD")

    def _git(self, command: str, *args):
        call_args = ["git", command]
        call_args.extend(args)
        return subprocess.run(
            call_args, stdout=subprocess.PIPE, universal_newlines=True
        ).stdout.rstrip()



from awesome_git_mosaic.usecases.write_mosaic import WriteMosaic


def write(
    text: str,
    strength: int = 5,
    multiply: int = 1,
    with_spaces: bool = True,
    background: bool = False,
    inverted: bool = False,
):
    WriteMosaic().write(
        text, strength, multiply, with_spaces, background, inverted
    )  # pragma: no cover
